#:coding=utf-8:

import re
import warnings
from types import StringType, UnicodeType

try:
    import json
except ImportError:
    import simplejson as json

from django.forms import CharField, RegexField, ValidationError
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import smart_unicode

from widgets import JSONWidget

__all__ = (
    'StripRegexField',
    'EmailField',
    'AlphaNumField',
    'NumCharField',
    'FullWidthCharField',
    'HiraganaCharField',
    'JSONField',
)

RE_EMAIL = re.compile(
    r"^[-\.!#$%&'*+/=?^_`{}|~0-9A-Z]+"  # account
    r"@(?:[A-Z0-9]+(?:-*[A-Z0-9]+)*\.)+[A-Z]{2,6}$",  # domain
    re.IGNORECASE
)
RE_ALPHA_NUM = re.compile(ur'^[a-zA-Z0-9\-_]*$')
RE_NUM = re.compile(ur'^[0-9]*$')
RE_FULL_WIDTH = re.compile(ur'[一-龠]+|[ぁ-ん]+|[ァ-ヴ]+|[０-９]+')
RE_HIRAGANA = re.compile(ur'^[ぁ-ゞー〜～＆ 　、・]*$')


class StripRegexField(RegexField):
    """
    検証する前にstripする正規表現のフィールド
    """
    def clean(self, value):
        if type(value) in (StringType, UnicodeType):
            value = value.strip()
        return super(StripRegexField, self).clean(value)


class EmailField(StripRegexField):
    default_error_messages = {
        'invalid': _(u'Eメールアドレスの形式が不正です'),
    }

    def __init__(self, *args, **kwargs):
        super(EmailField, self).__init__(RE_EMAIL, *args, **kwargs)


class AlphaNumField(StripRegexField):
    """
    半角英数字と"_","-"のみ許容するフィールド
    """
    default_error_messages = {
        'invalid': _(u'半角英数字で入力してください'),
    }

    def __init__(self, *args, **kwargs):
        super(AlphaNumField, self).__init__(RE_ALPHA_NUM, *args, **kwargs)


class NumCharField(StripRegexField):
    """
    数字のみを許容するフィールド
    """
    default_error_messages = {
        'invalid': _(u'数字で入力してください'),
    }

    def __init__(self, *args, **kwargs):
        super(NumCharField, self).__init__(RE_NUM, *args, **kwargs)


class FullWidthCharField(StripRegexField):
    """
    全角文字のみを許容するフィールド
    """
    default_error_messages = {
        'invalid': _(u'全角文字を入力してください。'),
    }

    def __init__(self, *args, **kwargs):
        super(FullWidthCharField, self).__init__(RE_FULL_WIDTH, *args, **kwargs)


class HiraganaCharField(StripRegexField):
    u"""
    全角ひらがなのみ許容するフィールド
    """
    default_error_messages = {
        'invalid': _(u'ひらがなで入力してください'),
    }

    def __init__(self, *args, **kwargs):
        super(HiraganaCharField, self).__init__(RE_HIRAGANA, *args, **kwargs)


class JSONField(CharField):
    u""" JSONデータをポストする場合のフィールド。AJAXに便利かも """
    default_error_messages = {
        'required': _('This field is required.'),
        'invalid': _('Enter a valid value.'),
    }

    def __init__(self, *args, **kwargs):
        warnings.warn('JSONField is deprecated. Use django-jsonfield instead.')
        if "widget" not in kwargs:
            kwargs["widget"] = JSONWidget
        super(JSONField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if value in ('', None):
            return u''
        if isinstance(value, basestring):
            return smart_unicode(value)
        else:
            return json.dumps(value)

    def clean(self, value):
        """
        Django 1.1 の場合、to_python()がないので、
        ここで、to_python() を呼び出して、super().clean()に渡す。
        Django 1.2 の場合、clean() の中に to_python() を
        ２重呼び出すが、２回呼び出しても、同じ結果になるのを
        保証するので、大丈夫。
        """
        value = super(JSONField, self).clean(self.to_python(value))
        if value in ('', None):
            return None

        try:
            json_data = json.loads(value)
        except Exception:
            raise ValidationError(self.error_messages['invalid'])
        return json_data



#:coding=utf-8:

from django.template import Library
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe

register = Library()

def stripentities(value):
    """Strips all HTML entities"""
    from django.utils.html import strip_entities
    return strip_entities(value)
stripentities.is_safe = True
stripentities = stringfilter(stripentities)
register.filter(stripentities)

@register.filter
def to_anchor(text, autoescape=None):
    from beproud.utils.html import urlize
    return mark_safe(urlize(text, attrs={"rel": "nofollow", "target": "_blank"}, autoescape=autoescape))
to_anchor.is_safe=True
to_anchor.needs_autoescape = True
to_anchor = stringfilter(to_anchor)

@register.filter
def to_anchortrunc(text, limit, autoescape=None):
    from beproud.utils.html import urlize
    return mark_safe(urlize(text, attrs={"rel": "nofollow", "target": "_blank"}, 
                     trim_url_limit=limit, autoescape=autoescape))
to_anchortrunc.is_safe=True
to_anchortrunc.needs_autoescape = True
to_anchortrunc = stringfilter(to_anchortrunc)

@register.filter
def force_js(value, type=None):
    from beproud.utils.javascript import force_js
    return mark_safe(force_js(value, type))



#:coding=utf-8:

from django.test import TestCase as DjangoTestCase

from beproud.django.commons.shortcuts import get_object_or_None, make_simple_response


class GetObjectOrNoneTestCase(DjangoTestCase):

    def _getTarget(self):
        from beproud.django.commons.tests.test_shortcuts.shortcuts_app.models import ShortcutModel
        return ShortcutModel

    def test_simple(self):
        target = self._getTarget()
        obj = get_object_or_None(target, pk=1)
        self.assertTrue(obj is not None)

    def test_queryset(self):
        target = self._getTarget()
        qs = target.objects.filter(name="queryset")
        obj = get_object_or_None(qs, pk=2)
        self.assertTrue(obj is not None)

    def test_none(self):
        target = self._getTarget()
        obj = get_object_or_None(target, pk=5)
        self.assertTrue(obj is None)


class MakeSimpleResponseTestCase(DjangoTestCase):

    def test_simple(self):
        response = make_simple_response()
        self.assertEqual(
            response.content,
            u'{"msg": "\\u51e6\\u7406\\u304c\\u6210\\u529f\\u3057\\u307e\\u3057\\u305f"}',
        )



# vim:fileencoding=utf-8
from django.core.cache import cache

__all__ = (
    'cache_get',
)

def cache_get(key, func, timeout=None, too_empty=False):
    """
    キャッシュにあれば使う、なければfuncを実行してキャッシュする
    too_empty=Trueにすると空の場合もキャッシュする
    """
    val = cache.get(key)
    if (not val) if too_empty else (val is None):
        val = func()
        cache.set(key, val, timeout)
    return val



#:coding=utf-8:

import warnings

try:
    import json
except ImportError:
    import simplejson as json

from django import forms

from beproud.django.commons.utils.javascript import DjangoJSONEncoder

__all__ = (
    'JSONWidget',
    'AdminJSONWidget',
)


class JSONWidget(forms.Textarea):
    def __init__(self, *args, **kwargs):
        warnings.warn('JSONWidget is deprecated. Use django-jsonfield instead.')
        self.indent = kwargs.pop("indent", 2)
        super(JSONWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        if not isinstance(value, basestring):
            value = json.dumps(value, indent=self.indent, cls=DjangoJSONEncoder)
        return super(JSONWidget, self).render(name, value, attrs)


class AdminJSONWidget(JSONWidget):
    def __init__(self, attrs=None):
        warnings.warn('AdminJSONWidget is deprecated. Use django-jsonfield instead.')
        final_attrs = {'class': 'vLargeTextField'}
        if attrs is not None:
            final_attrs.update(attrs)
        super(AdminJSONWidget, self).__init__(attrs=final_attrs)



import django.apps import AppConfig


class TestFieldsApp(AppConfig):
    name = 'fields'
    label = 'fields'


class TestShortcutsApp(AppConfig):
    name = 'shortcuts_app'
    label = 'shortcuts_app'


class TestBaseApp(AppConfig):
    name = 'base'
    label = 'base'



#:coding=utf-8:

from beproud.django.commons.test import URLTestCase

from beproud.django.commons.http import *

def test_json_response(request):
    return JSONResponse(data={"msg": u"成功しました"})

class JSONResponseTestCase(URLTestCase):
    url_list = (
        ('/test/json_response', {
            'check': ('assertOk', 'assertJson'),
        }),
    )



#:coding=utf-8:

from django.test import TestCase as DjangoTestCase

from beproud.django.commons.utils.cache import *

class CacheGetTestCase(DjangoTestCase):

    def test_cache_get(self):
        test_val = []
        def get_val():
            test_val.append("test")
            return "test-val"
        cached_val = cache_get("test-cache-get", get_val)
        self.assertEqual(cached_val, "test-val")
        self.assertEqual(test_val, ["test"])

        test_val = []
        cached_val = cache_get("test-cache-get", get_val)
        self.assertEqual(cached_val, "test-val")
        self.assertEqual(test_val, [])




#:coding=utf-8:

import base64
import warnings

try:
    import cPickle as pickle
except ImportError:
    import pickle

try:
    import json
except ImportError:
    import simplejson as json

from django import VERSION as DJANGO_VERSION
from django.core import exceptions
from django.utils.translation import ugettext_lazy as _
try:
   from django.db import load_backend
except ImportError:
   from django.db.utils import load_backend
from django.db.models import BigIntegerField

from django.db import models

from beproud.django.commons.forms import JSONField as JSONFormField
from beproud.django.commons.forms.widgets import JSONWidget
from beproud.django.commons.utils.javascript import DjangoJSONEncoder

__all__ = (
    'BigIntegerField',
    'PositiveBigIntegerField',
    'BigAutoField',
    'PickledObjectField',
    'JSONField',
)


class BigAutoField(models.AutoField):
    description = _("Big (8 byte) integer")

    CREATION_DATA = {
        'django.db.backends.mysql': "bigint AUTO_INCREMENT",
        'django.db.backends.oracle': "NUMBER(19)",
        'django.db.backends.postgresql': "bigserial",
        'django.db.backends.postgresql_psycopg2': "bigserial",
        'django.db.backends.sqlite3': "integer",  # NOTE: Not a bigint!!!
    }

    def db_type(self, connection):
        try:
            for backend, db_type in self.CREATION_DATA.items():
                try:
                    module = load_backend(backend)
                    DatabaseWrapper = getattr(module, 'DatabaseWrapper')
                    if isinstance(connection, DatabaseWrapper):
                        return db_type
                    elif DJANGO_VERSION >= (1, 7):
                        from django.db import DefaultConnectionProxy, connections
                        from django.db.utils import DEFAULT_DB_ALIAS
                        if (isinstance(connection, DefaultConnectionProxy) and

                                isinstance(connections[DEFAULT_DB_ALIAS], DatabaseWrapper)):
                            return db_type
                except (ImportError, exceptions.ImproperlyConfigured):
                    pass
        except (KeyError, AttributeError):
            pass
        raise exceptions.ImproperlyConfigured('BigAutoField does not support the "%s" database '
                                              'backend' % connection.settings_dict["ENGINE"])

    def get_internal_type(self):
        return "BigAutoField"

    def get_prep_value(self, value):
        if value is None:
            return None
        return long(value)

    def to_python(self, value):
        if value is None:
            return value
        try:
            return long(value)
        except (TypeError, ValueError):
            raise exceptions.ValidationError(self.error_messages['invalid'])


def fk_db_type(self, connection):
    # The database column type of a ForeignKey is the column type
    # of the field to which it points. An exception is if the ForeignKey
    # points to an AutoField/PositiveIntegerField/PositiveSmallIntegerField,
    # in which case the column type is simply that of an IntegerField.
    # If the database needs similar types for key fields however, the only
    # thing we can do is making AutoField an IntegerField.
    rel_field = self.rel.get_related_field()

    if (isinstance(rel_field, BigAutoField) or
            (not connection.features.related_fields_match_type and
             isinstance(rel_field, PositiveBigIntegerField))):
        return BigIntegerField().db_type(connection=connection)

    if (isinstance(rel_field, models.AutoField) or
            (not connection.features.related_fields_match_type and
             isinstance(rel_field, (models.PositiveIntegerField,
                                    models.PositiveSmallIntegerField)))):
        return models.IntegerField().db_type(connection=connection)
    return rel_field.db_type(connection=connection)

# ForeignKey monkey-patch to support BigAutoId
models.ForeignKey.db_type = fk_db_type


class PositiveBigIntegerField(BigIntegerField):
    # Note: Same internal type as BigIntegerField

    def formfield(self, **kwargs):
        defaults = {'min_value': 0}
        defaults.update(kwargs)
        return super(PositiveBigIntegerField, self).formfield(**defaults)


class PickledObjectField(models.TextField):
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        warnings.warn('PickledObjectField is deprecated. Use django-picklefield instead.')
        super(PickledObjectField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if value is None:
            return None
        if not isinstance(value, basestring):
            return value
        return pickle.loads(base64.b64decode(value))

    def get_db_prep_save(self, value, connection=None):
        if value is None:
            return
        return base64.b64encode(pickle.dumps(value))


class JSONField(models.TextField):
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        warnings.warn('JSONField is deprecated. Use django-jsonfield instead.')
        super(JSONField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'widget': JSONWidget, 'form_class': JSONFormField}
        defaults.update(kwargs)
        return super(JSONField, self).formfield(**defaults)

    def to_python(self, value):
        # If value is a bastring but empty then pass
        if isinstance(value, basestring):
            if value == '':
                return None
            else:
                value = json.loads(value)
        return value

    def get_db_prep_value(self, value, connection=None, prepared=None):
        if value is None:
            return
        return json.dumps(value, cls=DjangoJSONEncoder)

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value)


try:
    from south.modelsinspector import add_introspection_rules

    add_introspection_rules([

        ((BigAutoField,), [], {},),

    ], [r"^beproud\.django\.commons\.models"])
except ImportError:
    pass



#:coding=utf-8:

from django import template

__all__ = (
    'data_template_tag',
    'DataNode',
)

def data_template_tag(data_func, *args, **kwargs):
    """
    データを取ってくるタグ。

    data_funcはquerysetを返す関数

    使い方:
    @register.tag
    @data_template_tag
    def get_my_data():
        return MyData.objects.filter(...)

    テンプレートのほうは
    {% get_my_data as my_data %}
    
    引数も対応しています。

    @register.tag
    @data_template_tag
    def get_my_data(user):
        return MyData.objects.filter(user=user)

    {% get_my_data user as my_data %}

    キーワード引数も使えます
    """
    def wrapped(parser, token):
        bits = list(token.split_contents())

        args = []
        kwargs = {}
        name = None
        next_as = False 
        for bit in bits[1:]:
            if bit == 'as':
                next_as = True
            elif next_as:
                name = bit
                break
            else:
                if '=' in bit:
                    key, value = bit.split('=', 1)
                    if key and value:
                        kwargs[key] = value
                    else:
                        raise template.TemplateSyntaxError("invalid keyword arguments: %r" % bit)
                else:
                    args.append(bit)
                 
        if not name:
            raise template.TemplateSyntaxError("%r expected format is '%s *args **kwargs as <name>'" %
                                      (bits[0], bits[0]))
        return DataNode(data_func, args, kwargs, name)

    #名前を直す
    wrapped.__module__ = data_func.__module__
    wrapped.__name__ = data_func.__name__
    wrapped.__doc__ = data_func.__doc__
    return wrapped

class DataNode(template.Node):
    def __init__(self, data_func, args, kwargs, var_name):
        self.data_func = data_func 
        self.args = args
        self.kwargs = kwargs
        self.var_name = var_name

    def __repr__(self):
        return '<DataNode>'

    def render(self, context):
        args = [template.Variable(arg).resolve(context) for arg in self.args]
        kwargs = dict((str(key), template.Variable(value).resolve(context)) for key, value in self.kwargs.iteritems())
        context[self.var_name] = self.data_func(*args, **kwargs)
        return u''



#:coding=utf-8:

try:
    import json
except ImportError:
    import simplejson as json

from types import StringType, UnicodeType
from django.test import TestCase

__all__ = (
    'InvalidTest',
    'RequestTestCase',
    'URLTestCase',
)


class InvalidTest(Exception):
    def __init__(self, msg=''):
        self.msg = msg

    def __str__(self):
        return 'InvalidTest: %s' % self.msg


class RequestTestCase(TestCase):

    def assertStatus(self, response, status=200):
        self.assertEquals(response.status_code, status)

    def assertOk(self, response):
        self.assertStatus(response)

    def assertCreated(self, response):
        self.assertStatus(response, 201)

    def assertAccepted(self, response):
        self.assertStatus(response, 202)

    def assertNonAuthoritativeInfo(self, response):
        self.assertStatus(response, 203)

    def assertNoContent(self, response):
        self.assertStatus(response, 204)

    def assertResetContent(self, response):
        self.assertStatus(response, 205)

    def assertPartialContent(self, response):
        self.assertStatus(response, 206)

    def assertMultipleChoices(self, response):
        self.assertStatus(response, 300)

    def assertPermanentRedirect(self, response, redirect_url=None):
        self.assertStatus(response, 301)
        self._assertLocationHeader(response, redirect_url)

    def assertRedirect(self, response, redirect_url=None):
        self.assertStatus(response, 302)
        self._assertLocationHeader(response, redirect_url)

    def assertBadRequest(self, response):
        self.assertStatus(response, 400)

    def assertForbidden(self, response):
        self.assertStatus(response, 403)

    def assertNotFound(self, response):
        self.assertStatus(response, 404)

    def assertNotAllowed(self, response, allow=None):
        self.assertEquals(response.status_code, 405)
        if allow is not None:
            self.assertEquals(response["Allow"], allow)

    def assertGone(self, response):
        self.assertEquals(response.status_code, 410)

    def assertHtml(self, response):
        self.assertContains(response, "<html")  # open tag
        self.assertContains(response, "</html>")  # close tag
        self.assertContains(response, "<head")
        self.assertContains(response, "</head>")
        self.assertContains(response, "<body")
        self.assertContains(response, "</body>")

    def assertJson(self, response):
        try:
            return json.loads(response.content)
        except ValueError, e:
            self.fail(e.message)

    def assertXml(self, response):
        from xml.parsers import expat
        try:
            p = expat.ParserCreate()
            return p.Parse(response.content)
        except expat.ExpatError, e:
            self.fail(e.message)

    def _assertLocationHeader(self, response, redirect_url=None):
        if redirect_url is None:
            self.assertTrue(response.get("Location", None) is not None)
        else:
            self.assertEquals(response.get("Location", None), redirect_url)


class BaseURLTestCase(type):
    def __new__(cls, name, bases, attrs):
        counter = 0
        _username = attrs.get('username')
        _password = attrs.get('password')
        for data in attrs['url_list']:
            # default
            name = ''
            url = ''
            headers = {}
            check = ('assertOk',)
            method = 'get'
            username = _username
            password = _password
            urlparams = {}

            # url only
            if type(data) in (StringType, UnicodeType):
                url = data
            else:
                # url
                if len(data) < 0:
                    raise InvalidTest('"%s"' % data)
                url = data[0]
                # options
                if len(data) > 1:
                    options = data[1]
                    # check
                    if 'check' in options:
                        if type(options['check']) in (StringType, UnicodeType):
                            check = (options['check'], )
                        else:
                            check = options['check']
                    # name
                    if 'name' in options:
                        name = '_%s' % options['name']
                    if 'headers' in options:
                        headers = options['headers']
                    # method
                    if 'method' in options:
                        method = options['method']
                    # login
                    if 'username' in options:
                        username = options['username']
                    if 'password' in options:
                        password = options['password']
                    # urlparams
                    if 'urlparams' in options:
                        urlparams = options['urlparams']

            def _outer(url, check, method, headers, username, password, urlparams):
                def _url_test(self):
                    if username:
                        self.assertTrue(self.client.login(username=username, password=password))
                    response = getattr(self.client, method)(url, urlparams, **headers)
                    for check_options in check:
                        # check method
                        if type(check_options) in (StringType, UnicodeType):
                            getattr(self, check_options)(response)
                        else:
                            _method = getattr(self, check_options[0])
                            if len(check_options) > 1:
                                args = check_options[1]
                                if len(check_options) > 2:
                                    kwargs = check_options[2]
                                    _method(response, *args, **kwargs)
                                else:
                                    _method(response, *args)
                            else:
                                _method(response)
                return _url_test

            attrs['test_url_%d%s' % (counter, name)] = _outer(url, check, method, headers, username, password, urlparams)
            counter += 1
        return type.__new__(cls, name, bases, attrs)


class URLTestCase(RequestTestCase):
    """
    URLに対してGET/POSTを実行してレスポンスを確認する

    url_list = (
        ('/foo/bar', {
            'check': ('assertOk', ('assertJson', [], {})),
            'method': 'post',
            'urlparams': {'param1': 'value1',
            'username': 'user1',
            'password': 'pass1'}
        }),
        r'/',
    )
    """
    username = ''
    password = ''
    url_list = ()
    __metaclass__ = BaseURLTestCase



#:coding=utf-8:

from django.db import models

from beproud.django.commons.models import (
    BaseModel,
    BigAutoField,
    JSONField,
)


class BigIDModel(BaseModel):
    id = BigAutoField(primary_key=True)
    class Meta:
        app_label = 'fields'


class TestBigIntModel(BaseModel):
    big_id_obj = models.ForeignKey(BigIDModel)
    class Meta:
        app_label = 'fields'


class SmallIDModel(BaseModel):
    class Meta:
        app_label = 'fields'


class TestBigToSmallModel(BaseModel):
    small_id_obj = models.ForeignKey(SmallIDModel)
    class Meta:
        app_label = 'fields'


class ManyToManyTestModel(BaseModel):
    bigids = models.ManyToManyField(BigIDModel)
    class Meta:
        app_label = 'fields'


class JSONFieldTestModel(models.Model):
    json = JSONField(u"test", null=True, blank=True)
    class Meta:
        app_label = 'base'



#:coding=utf-8:

from functools import wraps

from django.shortcuts import render_to_response
from django.template import RequestContext

from beproud.django.commons.http import JSONResponse

def render_to(template=None):
    """
    Decorator for Django views that sends returned dict to render_to_response function.

    Template name can be decorator parameter or TEMPLATE item in returned dictionary.
    RequestContext always added as context instance.
    If view doesn't return dict then decorator simply returns output.

    Parameters:
     - template: template name to use

    Examples:
    # 1. Template name in decorator parameters

    @render_to('template.html')
    def foo(request):
        bar = Bar.object.all()  
        return {'bar': bar}

    # equals to 
    def foo(request):
        bar = Bar.object.all()  
        return render_to_response('template.html', 
                                  {'bar': bar}, 
                                  context_instance=RequestContext(request))


    # 2. Template name as TEMPLATE item value in return dictionary

    @render_to()
    def foo(request, category):
        template_name = '%s.html' % category
        return {'bar': bar, 'TEMPLATE': template_name}
    
    #equals to
    def foo(request, category):
        template_name = '%s.html' % category
        return render_to_response(template_name, 
                                  {'bar': bar}, 
                                  context_instance=RequestContext(request))

    Taken from django-annoying
    """
    def renderer(function):
        @wraps(function)
        def wrapper(request, *args, **kwargs):
            output = function(request, *args, **kwargs)
            if not isinstance(output, dict):
                return output
            tmpl = output.pop('TEMPLATE', template)
            return render_to_response(tmpl, output, context_instance=RequestContext(request))
        return wrapper
    return renderer

def ajax_request(func):
    """
    If view returned serializable dict, returns JSONResponse with this dict as content.

    example:
        
        @ajax_request
        def my_view(request):
            news = News.objects.all()
            news_titles = [entry.title for entry in news]
            return {'news_titles': news_titles}

    Taken from django-annoying
    """
    @wraps(func)
    def wrapper(request, *args, **kwargs):
        response = func(request, *args, **kwargs)
        if isinstance(response, (dict, list, tuple)):
            return JSONResponse(response)
        else:
            return response
    return wrapper



#:coding=utf-8:

import warnings

from django.template import (
    Library,
    Node, 
    TemplateSyntaxError,
    VariableDoesNotExist,
)

register = Library()

@register.tag(name="switch")
def do_switch(parser, token):
    """
    The ``{% switch %}`` tag compares a variable against one or more values in
    ``{% case %}`` tags, and outputs the contents of the matching block.  An
    optional ``{% else %}`` tag sets off the default output if no matches
    could be found::

        {% switch result_count %}
            {% case 0 %}
                There are no search results.
            {% case 1 %}
                There is one search result.
            {% else %}
                Jackpot! Your search found {{ result_count }} results.
        {% endswitch %}

    Each ``{% case %}`` tag can take multiple values to compare the variable
    against::

        {% switch username %}
            {% case "Jim" "Bob" "Joe" %}
                Me old mate {{ username }}! How ya doin?
            {% else %}
                Hello {{ username }}
        {% endswitch %}
    """
    warnings.warn('The switch template tag is deprecated. Use {% if ... %}...{% elif ... %}...{% endif %} (Django >1.4)')

    bits = token.contents.split()
    tag_name = bits[0]
    if len(bits) != 2:
        raise TemplateSyntaxError("'%s' tag requires one argument" % tag_name)
    variable = parser.compile_filter(bits[1])

    class BlockTagList(object):
        # This is a bit of a hack, as it embeds knowledge of the behaviour
        # of Parser.parse() relating to the "parse_until" argument.
        def __init__(self, *names):
            self.names = set(names)
        def __contains__(self, token_contents):
            name = token_contents.split()[0]
            return name in self.names

    # Skip over everything before the first {% case %} tag
    parser.parse(BlockTagList('case', 'endswitch'))

    cases = []
    token = parser.next_token()
    got_case = False
    got_else = False
    while token.contents != 'endswitch':
        nodelist = parser.parse(BlockTagList('case', 'else', 'endswitch'))
        
        if got_else:
            raise TemplateSyntaxError("'else' must be last tag in '%s'." % tag_name)

        contents = token.contents.split()
        token_name, token_args = contents[0], contents[1:]
        
        if token_name == 'case':
            tests = map(parser.compile_filter, token_args)
            case = (tests, nodelist)
            got_case = True
        else:
            # The {% else %} tag
            case = (None, nodelist)
            got_else = True
        cases.append(case)
        token = parser.next_token()

    if not got_case:
        raise TemplateSyntaxError("'%s' must have at least one 'case'." % tag_name)

    return SwitchNode(variable, cases)

class SwitchNode(Node):
    def __init__(self, variable, cases):
        self.variable = variable
        self.cases = cases

    def __repr__(self):
        return "<Switch node>"

    def __iter__(self):
        for tests, nodelist in self.cases:
            for node in nodelist:
                yield node

    def get_nodes_by_type(self, nodetype):
        nodes = []
        if isinstance(self, nodetype):
            nodes.append(self)
        for tests, nodelist in self.cases:
            nodes.extend(nodelist.get_nodes_by_type(nodetype))
        return nodes

    def render(self, context):
        try:
            value_missing = False
            value = self.variable.resolve(context, True)
        except VariableDoesNotExist:
            no_value = True
            value_missing = None
        
        for tests, nodelist in self.cases:
            if tests is None:
                return nodelist.render(context)
            elif not value_missing:
                for test in tests:
                    test_value = test.resolve(context, True)
                    if value == test_value:
                        return nodelist.render(context)
        else:
            return ""



#:coding=utf-8:

# NOTE: Django 1.6 以上の場合はテストランナーは test*.py
# というパターンにマッチするファイルしか認識しないので、
# このファイル (__init__.py) に入っているテストは認識しない。
# このファイルは主に Django 1.5以下のテストランナーのために
# テストをインポートしています。
import django

if django.VERSION < (1, 9):
    from beproud.django.commons.tests.test_javascript import *  # NOQA
    from beproud.django.commons.tests.test_template import *  # NOQA
    from beproud.django.commons.tests.test_cache import *  # NOQA
    from beproud.django.commons.tests.test_views import *  # NOQA
    from beproud.django.commons.tests.test_templatetags import *  # NOQA
    from beproud.django.commons.tests.test_http import *  # NOQA
    from beproud.django.commons.tests.test_shortcuts import *  # NOQA

    from beproud.django.commons.tests.models.base.tests import *  # NOQA
    from beproud.django.commons.tests.models.fields.tests import *  # NOQA

    from beproud.django.commons.tests.forms.test_field import *  # NOQA



#:coding=utf-8:

from django.conf.urls import patterns, include

from beproud.django.commons.tests.test_views import TestViews

urlpatterns = patterns(
    '',
    (r'', include(TestViews().urls)),
    (r'json_response', 'beproud.django.commons.tests.test_http.test_json_response'),
)



#:coding=utf-8:

from beproud.django.commons.models import DatedModel, BaseModel


class TestDatedModel(DatedModel):
    class Meta:
        app_label = 'base'


class TestBaseModel(BaseModel):
    class Meta:
        app_label = 'base'



#:coding=utf-8:

from django import VERSION as DJANGO_VERSION
from django.test import TestCase as DjangoTestCase
from django.template import Template, TemplateSyntaxError
try:
    from django.template import (
        Lexer,
        Parser,
    )
except ImportError:
    from django.template.base import (
        Lexer,
        Parser,
    )

from django.template.loader import LoaderOrigin
from django.template.context import Context


class BaseTemplateTagTest(object):

    def _make_origin(self):
        return LoaderOrigin("Commons Test", lambda x,y: ("<string>", "<string>"), "commons", [])

    def _render_html(self, template_string, context={}):
        # :(
        if DJANGO_VERSION > (1,9):
            from django.template.library import import_library
            tag_lib = import_library('beproud.django.commons.tests.test_tags')
        elif DJANGO_VERSION > (1,7):
            from django.template.base import import_library
            tag_lib = import_library('beproud.django.commons.tests.test_tags')
        elif DJANGO_VERSION > (1,2):
            from django.template import import_library
            tag_lib = import_library('beproud.django.commons.tests.test_tags')
        else:
            from django.template import get_library
            tag_lib = get_library('beproud.django.commons.tests.test_tags')

        if DJANGO_VERSION > (1,9):
            lexer = Lexer(template_string)
        else:
            lexer = Lexer(template_string, self._make_origin())
        parser = Parser(lexer.tokenize())
        parser.add_library(tag_lib)
        nodelist = parser.parse()

        return nodelist.render(Context(context))

class DataTemplateTagTestCase(BaseTemplateTagTest, DjangoTestCase):

    TEMPLATE_STRING = "<html><body>{% get_my_data 121 as my_data %}{{ my_data }}</body></html>"
    BAD_TEMPLATE_STRING = "<html><body>{% get_my_data 121 my_data %}{{ my_data }}</body></html>"

    def test_data_template_tag(self):
        self.assertEquals(self._render_html(self.TEMPLATE_STRING), "<html><body>MY DATA</body></html>")

    def test_bad_template_tag(self):
        try:
            html = self._render_html(self.BAD_TEMPLATE_STRING)
            self.fail("Expected Fail: %s" % html)
        except TemplateSyntaxError, e:
            pass

class KwargDataTemplateTagTestCase(BaseTemplateTagTest, DjangoTestCase):

    TEMPLATE_STRING1 = "<html><body>{% get_my_kwarg_data 121 as my_data %}{{ my_data }}</body></html>"
    TEMPLATE_STRING2 = "<html><body>{% get_my_kwarg_data 121 status='spam' as my_data %}{{ my_data }}</body></html>"
    TEMPLATE_STRING3 = "<html><body>{% get_my_kwarg_data 121 other='eggs' as my_data %}{{ my_data }}</body></html>"
    TEMPLATE_STRING4 = "<html><body>{% get_my_kwarg_data 121 status='spam' other='eggs' as my_data %}{{ my_data }}</body></html>"
    TEMPLATE_STRING5 = "<html><body>{% get_my_kwarg_data 121 status=spam other=eggs as my_data %}{{ my_data }}</body></html>"

    BAD_TEMPLATE_STRING1 = "<html><body>{% get_my_kwarg_data 121 my_data %}{{ my_data }}</body></html>"
    BAD_TEMPLATE_STRING2 = "<html><body>{% get_my_kwarg_data %}{{ my_data }}</body></html>"
    BAD_TEMPLATE_STRING3 = "<html><body>{% get_my_kwarg_data as my_data %}{{ my_data }}</body></html>"

    BASE_HTML = "<html><body>%s</body></html>"

    def test_kwarg_data_template_tag1(self):
        self.assertEquals(self._render_html(self.TEMPLATE_STRING1), self.BASE_HTML % "121:None:other")

    def test_kwarg_data_template_tag2(self):
        self.assertEquals(self._render_html(self.TEMPLATE_STRING2), self.BASE_HTML % "121:spam:other")

    def test_kwarg_data_template_tag3(self):
        self.assertEquals(self._render_html(self.TEMPLATE_STRING3), self.BASE_HTML % "121:None:eggs")

    def test_kwarg_data_template_tag4(self):
        self.assertEquals(self._render_html(self.TEMPLATE_STRING4), self.BASE_HTML % "121:spam:eggs")

    def test_kwarg_data_template_tag5(self):
        self.assertEquals(self._render_html(self.TEMPLATE_STRING5, {"spam": "eggs", "eggs": "spam"}), self.BASE_HTML % "121:eggs:spam")

    def test_bad_template_tag1(self):
        try:
            html = self._render_html(self.BAD_TEMPLATE_STRING1)
            self.fail("Expected Fail: %s" % html)
        except TemplateSyntaxError, e:
            pass

    def test_bad_template_tag2(self):
        try:
            html = self._render_html(self.BAD_TEMPLATE_STRING2)
            self.fail("Expected Fail: %s" % html)
        except TemplateSyntaxError, e:
            pass



#:coding=utf-8:

from django.db import models

from beproud.django.commons.models import BaseModel


class ShortcutModel(BaseModel):
    name = models.CharField(u"Name", max_length=20)
    class Meta:
        app_label = 'shortcuts_app'



#:coding=utf-8:
import os

from django.test import TestCase as DjangoTestCase

from beproud.django.commons.tests.models.base.models import TestDatedModel, TestBaseModel


here = os.path.dirname(__file__)


class DatedModelTest(DjangoTestCase):
    fixtures = [os.path.join(here, 'fixtures', 'initial_data.json')]

    def test_simple(self):
        qs = TestDatedModel.objects.recently_updated()

        # クエリがデータを返すかどうかをチェック
        self.assertTrue(qs)

        previous = None
        for obj in qs:
            if previous:
                self.assertTrue(obj.utime <= previous.utime)
            previous = obj


class BaseModelTest(DjangoTestCase):
    fixtures = [os.path.join(here, 'fixtures', 'initial_data.json')]

    def test_be(self):
        qs = TestBaseModel.objects.be()

        for obj in qs:
            self.assertTrue(not obj.del_flg)

    def test_remove(self):
        obj = TestBaseModel.objects.get(pk=1)
        self.assertTrue(not obj.del_flg)

        obj.remove()
        self.assertTrue(obj.del_flg)

        obj_updated = TestBaseModel.objects.get(pk=1)
        self.assertTrue(obj_updated.del_flg)

    def test_unremove(self):
        obj = TestBaseModel.objects.get(pk=3)
        self.assertTrue(obj.del_flg)

        obj.unremove()
        self.assertTrue(not obj.del_flg)

        obj_updated = TestBaseModel.objects.get(pk=3)
        self.assertTrue(not obj_updated.del_flg)



# vim:fileencoding=utf-8

try:
    import json
except ImportError:
    import simplejson as json

from django.http import HttpResponse

from beproud.django.commons.utils.javascript import DjangoJSONEncoder

__all__ = (
    'JSONResponse',
)


class JSONResponse(HttpResponse):
    """
    HttpResponse descendant, which return response with ``application/json`` mimetype.
    """
    def __init__(self, data={}, status=200, content_type='application/json'):
        super(JSONResponse, self).__init__(
            json.dumps(data, cls=DjangoJSONEncoder),
            status=status,
            content_type=content_type,
        )
